import { Span } from 'opentracing';
import 'reflect-metadata';
import Base from './base';
import { default as broker } from './brokers';
import { MsError } from './Error';
import { ConfService, DataRequest, ResponseMessage } from './types';
export declare abstract class Microservice extends broker {
    /**
     * Aplicacion del resultado.
     * @protected
     */
    protected app: Base;
    /**
     * Coleccion de errores
     * @private
     * @type Map
     */
    private ErrorColl;
    /**
     * Coleccion de errores custom
     * @private
     * @type Map
     */
    private ErrorCustomColl;
    /**
     * Coleccion de Forks (bifurcaciones)
     * @private
     * @type Map
     */
    private ForksColl;
    /**
     * Hostname del nodo
     * @private
     */
    private hostname;
    /**
     * Version del framework
     * @private
     */
    private frameworkVersion;
    /**
     * Contructor de la clase
     * @param conf Configuracion del servicio
     */
    constructor(conf: ConfService);
    /**
     * Metodo para el procesamiento de mensajes
     * @param data mensaje
     */
    message(data: DataRequest): Promise<any>;
    /**
     * Metodo para publicar un error en la cola de mensajes
     * @param service Nombre del servicio
     * @param transactionId Id de la transaccion
     * @param error Error
     */
    sendError(service: string, transactionId: string | number, error: any): void;
    /**
     * Metodo para publicar la respuesta
     *
     * @param {DataRequest} data
     * @param {(number | ResponseMessage)} error
     * @param {Span} [span]
     * @param {boolean} [smoketest=false]
     * @memberof Microservice
     */
    response(data: DataRequest, error: number | ResponseMessage | any, span?: Span, smoketest?: boolean): void;
    /**
     * Metodo que se ejecuta al ocurrir una excepcion al procesar mensajes.
     * @param topic Topico del Error
     * @param data Mensaje de la peticion
     * @param error Error que se ejecuta
     */
    exception(topic: string, data: string, error: Error): void;
    /**
     * Metodo para realizar la prueba de smoke Test
     * @public
     * @abstract
     */
    abstract smoketest(): boolean | Promise<boolean>;
    /**
     * Metodo para validar que la seguridad necesaria
     * sea valida.
     * @param token Token de seguridad
     */
    auth(token: string): Promise<boolean>;
    /**
     * Metodo que se ejecuta en el healthcheck
     * @publick
     */
    HealthCheck(): boolean | Promise<boolean>;
    /**
     * Metodo para verificar si existe un codigo de error
     * ya registrado.
     * @param code Codigo del error
     */
    hasError(code: number): boolean;
    /**
     * Metodo para agregar nuevos errores a la coleccion.
     * @param text Texto del error
     * @param code Codigo del error
     */
    addError(text: string, code: number): void;
    /**
     * Metodo para generar una excepcion en el codigo
     * @param code Codigo de error
     */
    protected emitError(code: number, message?: string): MsError;
    /**
     * Metodo para iniciar el HealthCheck HTTP
     * @private
     */
    private startHttpHealthCheck;
    /**
     * Metodo para recuperar un Error.
     * @param code Codigo de Error
     */
    private getError;
    /**
     * Metodo para recuperar la configuracion de un elemento
     * registrado con los decoradores.
     * @param Name Nombre de la configuracion
     */
    private getConfHandler;
    /**
     * Metodo para resolver la definicion que se ejecutara.
     * @param name Nombre de la definicion que se busca
     */
    private resolveDefinition;
    /**
     * Metodo para recuperar los datos de configuracion para una
     * accion.
     * @param NameAction Nombre de la accion
     */
    private getConfForks;
    /**
     * Metodo para recuperar los datos de un Fork
     * @param name Nombre del fork definido
     */
    private getConfDefinitionFork;
    /**
     * Metodo para recuperar los parametros requeridos
     * para un metodo.
     * @param name Nombre del metodo
     */
    private getParams;
    /**
     * Metodo para realizar la busqueda de una bifurcacion.
     * @param Col Coleccion de callbacks
     * @param find Bifurcacion que es buscada
     */
    private existCallback;
}
