"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const EventEmitter = require("events");
const os = require("os");
const logs_1 = require("./logs");
const nanoid_1 = require("nanoid");
const jaeger_client_1 = require("jaeger-client");
// Middleware
const cors = require("cors");
const bodyparser = require("body-parser");
/**
 * Clase para crear un transport, remplazando un broker.
 *
 * @export
 * @class HttpTransport
 * @extends {EventEmitter}
 */
class HttpTransport extends EventEmitter {
    /**
     * Creates an instance of HttpTransport.
     * @param {string} topic
     * @memberof HttpTransport
     */
    constructor(topic, version, appname, // Nombre de la aplicacion
    serviceName) {
        super();
        /**
         * Instancia de express
         *
         * @private
         * @type {express.Application}
         * @memberof HttpTransport
         */
        this.app = express();
        /**
         * Puerto en el que inicia el servidor http
         *
         * @private
         * @type {number}
         * @memberof HttpTransport
         */
        this.port = process.env.HTTP_PORT
            ? parseInt(process.env.HTTP_PORT, 0)
            : 3000;
        /**
         * Manejador de logs
         *
         * @private
         * @type {*}
         * @memberof HttpTransport
         */
        this.logs = logs_1.default;
        /**
         * Topico del servicio
         *
         * @private
         * @type {string}
         * @memberof HttpTransport
         */
        this.topicName = '';
        this.topic = topic;
        this.hostname = os.hostname();
        this.appname = appname;
        this.version = version;
        this.serviceName = serviceName;
    }
    /**
     * Metodo para crear los datos necesarios de la metadata.
     *
     * @private
     * @returns {Metadata}
     * @memberof HttpTransport
     */
    createMetaData(operationID, topicName, topic, transactionID, hostname, asynchronous, id, method, deviceType, filters, span) {
        return {
            uuid: id,
            id_operacion: operationID,
            id_transaction: transactionID,
            intents: 1,
            callback: topic,
            callbacks: [{ method, name: topicName }],
            owner: topic,
            uowner: topic,
            uworker: `${topic}_${operationID}`,
            worker: topic,
            asynchronous,
            mtype: 'input',
            bifurcacion: false,
            time: new Date().toISOString(),
            from: hostname,
            method,
            deviceType,
            span,
            filters,
        };
    }
    /**
     * Metodo para ejecutar una peticion.
     *
     * @private
     * @param {DataRequest} request
     * @returns {Promise<DataRequest>}
     * @memberof HttpTransport
     */
    process(request) {
        return new Promise((rr, ff) => {
            // Publicar el mensaje
            this.emit('message', request);
            // Escuchar la respuesta
            this.once(request.metadata.id_transaction, (resp) => {
                // Retornar la respuesta
                rr(resp);
            });
        });
    }
    /**
     * Metodo para preparar los datos requeridos de la peticion
     *
     * @private
     * @param {string} method
     * @param {*} data
     * @param {*} filters
     * @param {string} uuid
     * @param {*} headers
     * @returns {DataRequest}
     * @memberof HttpTransport
     */
    prepare(method, data, filters, uuid, headers, span, operationID) {
        // const message = Buffer.from(JSON.stringify({}),).toString('base64')
        const transactionID = nanoid_1.nanoid();
        // Retornar los datos necesarios.
        return {
            metadata: this.createMetaData(operationID, this.topicName, this.topicName, transactionID, this.hostname, false, uuid, method, 'PC', filters, span),
            data,
            headers,
        };
    }
    /**
     * Handler para las peticiones desde el cliente.
     *
     * @private
     * @param {express.Request} req
     * @param {express.Response} resp
     * @memberof HttpTransport
     */
    handler(req, resp) {
        return __awaiter(this, void 0, void 0, function* () {
            // Tiempo inicial de la operacion
            const start = new Date().getTime();
            // Id unico de la operacion
            const operationID = new Date().getTime() / 1000000 + '';
            // Id de la operacion
            const id = req.params.id || null;
            // Metodo
            const method = req.method;
            // Filters
            const filters = req.query;
            // Data
            const data = req.body || {};
            // Headers
            const headers = req.headers;
            // Iniciar el traecing
            const tracer = jaeger_client_1.initTracer({
                serviceName: `${this.topic}`,
            }, {
                tags: {
                    transactionId: operationID,
                },
            });
            // Generar el span
            const span = tracer.startSpan(`${this.appname}-${this.serviceName}`);
            // Crear la consulta
            const reqOperation = this.prepare(method, data, filters, id, headers, span, operationID);
            try {
                const respOperation = yield this.process(reqOperation);
                const r = respOperation.response;
                // Validar si ocurrio un error
                if (r.data.response.errorCode || r.data.response.errorCode < 0) {
                    resp.status(500).json(respOperation.response);
                }
                else {
                    resp.json(respOperation.response);
                }
            }
            catch (error) {
                resp.status(500).json({
                    meta: {
                        status: 'ERROR',
                        id_transaction: operationID,
                        time_elapsed: new Date().getTime() - start,
                    },
                    data: {
                        response: {
                            errorCode: -9,
                            userMessage: '-35',
                        },
                    },
                });
            }
            finally {
                // Finalizar el span
                span.finish();
            }
        });
    }
    /**
     *  Metodo que inicia el servidor HTTP.
     *  @public
     */
    start() {
        // Escuchar los eventos
        const routes = [
            `/${this.appname}/api/${this.version}/${this.serviceName}`,
            `/${this.appname}/api/${this.version}/${this.serviceName}/:id`,
            `/api/${this.version}/${this.serviceName}`,
            `/api/${this.version}/${this.serviceName}/:id`,
        ];
        // Asignar los middlewares
        this.app.use(cors());
        // this.app.use(audit());
        this.app.use(bodyparser.json());
        this.app.use(bodyparser.urlencoded({ extended: false }));
        // Asignar las rutas
        routes.forEach((route) => {
            this.app.all(route, this.handler.bind(this));
        });
        // Iniciar server
        this.app.listen(this.port, '0.0.0.0', () => {
            this.logs.info(`Se inicia el servidor http en el puerto ${this.port}`);
        });
    }
}
exports.default = HttpTransport;
